# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from pypgmtools.timing import controller, modifier, keyframe
from pypgmtools.graph.group import Group
from pypgmtools.graph.image import Image
from pypgmtools.timing import implicit
import pgm
import os


class Slideshow(Group):

    def __init__(self, canvas, list_of_files = []):
        """
        the Slideshow contains a list of images (path to files)

        @param canvas:          the canvas to paint on
        @param list_of_files:   a list of files, which should be shown. e.g.:
                                ['path/to/fileone', 'path/to/filetwo' ... ]
        @type list_of_files:    list
        """

        Group.__init__(self, canvas)

        self._list = list_of_files
        self._index = 0

        self._image = Image()
        self._image.bg_color = 0,0,0,0
        self._image.height = canvas.height * 0.99
        self._image.width = canvas.width * 0.99
        self._image.visible = True

        self._next = None
        self._prev = None
        self._ani_img = implicit.AnimatedObject(self._image)

        if not len(self._list) == 0:
            self._load_other()
            self._paint_image(self._image)

        self.add(self._image)


    def list__set(self, list_of_files):
        """
        Set the list of files to list_of_files.

        @param list_of_files:   list of files
        @type list_of_files:    list
        Attenton: We set the index to 0 again.
        We are not painting automatically, please use index__set!
        """

        if isinstance(list_of_files, list):
            self._list = list_of_files
            self._index = 0

    def list__get(self):
        """
        Returns the list
        """
        return self._list


    def next(self):
        """
        Show the next image (automatically fade in and out)
        """
        if self._index < (len(self._list) - 1):
            self._index = self._index + 1
            self._paint_image(self._next)
            self._load_other()

    def _load_other(self):
        """
        We are caching the next one and the previous one for
        faster usage
        """
        if self._index < self.length - 1:
            del self._next
            self._next = Image()
            self._next.bg_color = (0, 0, 0, 0)
            self._next.visible = True
            self._next.opacity = 0
            self._next.set_from_fd(os.open(self._list[self._index+1],
                                    os.O_RDONLY))
            self._next.position = (0.0, 0.0, 0.0)
        if self._index > 0:
            del self._prev
            self._prev = Image()
            self._prev.bg_color = (0, 0, 0, 0)
            self._prev.visible = True
            self._prev.opacity = 0
            self._prev.set_from_fd(os.open(self._list[self._index-1],
                                    os.O_RDONLY))
            self._prev.position = (0.0, 0.0, 0.0)

    def previous(self):
        """
        Show the previous image (automatically fade in and out)
        """
        if self._index > 0:
            self._index = self._index - 1
            self._paint_image(self._prev)
            self._load_other()


    def length__get(self):
        """
        Returns the length of our list
        """
        return len(self._list)

    def index__set(self, index):
        """
        Set the index to index.
        @type index:    int
        """
        if isinstance(index, int):
            if index < len(self._list) and index >= 0:
                self._index = index - 1
                self._load_other()
                self.next()


    def index__get(self):
        """
        Returns the index-number (which is the current position in the list)
        """
        return self._index


    def _paint_image(self, new_img):
        """
        internal method, which paints the image
        """
        if new_img:
            self._ani_img.opacity = 0
            new_img.visible = True
            new_img.opacity = 0
            new_img.set_from_fd(os.open(self._list[self._index], os.O_RDONLY))
            new_img.width = self._image.width
            new_img.height = self._image.height
            new_img.position = (0.0, 0.0, 0.0)
            ani_new = implicit.AnimatedObject(new_img)
            ani_new.setup_next_animations(transformation = implicit.DECELERATE)
            ani_new.opacity = 255
            self.add(new_img)
            self._ani_img = ani_new


from twisted.internet import reactor
from twisted.internet.error import AlreadyCalled, AlreadyCancelled


class TwistedSlideshow(Slideshow):

    """
    This class implements a timed slideshow using the twisted framework
    """

    def __init__(self, canvas, list_of_files, delay_time):
        """
        We need the canvas on which we should paint on and the list of files (see
        Slideshow for it)
        @param canvas: the canvas to paint on
        @oaram list_of_files: please have a look at the init of Slideshow
        @param delay_time: the delay time between calling the next one
        @type delay_time int which represents seconds

        Attention: the Slideshow is not starting on it's own. Use start() to
        start it
        """
        Slideshow.__init__(self, canvas, list_of_files)
        self._delaytime = delay_time
        self._timer = None
        self._active = False

    def delay_time__set(self, seconds):
        """
        Set the delay time to seconds
        """
        self._delaytime = seconds
        if self._timer:
            selt._timer.reset(seconds)

    def call_time__get(self):
        """
        Return the time when the next image is shown (float of seconds since epoch)
        """
        if self._timer:
            return self._timer.getTime()
        return 0

    def delay_time__get(self):
        """
        Get the delay time
        """
        return self._delaytime

    def start(self):
        """
        Start the slideshow (he get visible now, if we are not yet!)
        """
        self._active = True
        self.visible = True
        self._timer = reactor.callLater(self._delaytime, self.next)

    def active__get(self):
        """
        The the slideshow timer still active
        """
        return self._active


    def stop(self):
        """
        Stop the slideshow (which does mean, that it is not going on, but it
        is still visible!)
        """
        self._active = False
        if self._timer:
            try:
                self._timer.cancel()
            except (AlreadyCalled, AlreadyCancelled):
                pass
            self._timer = None

    def _reset_timer(self):
        if self._timer:
            try:
                self._timer.reset(self._delaytime)
            except (AlreadyCalled, AlreadyCancelled):
                self._timer = None

        if self._active:
            self._timer = reactor.callLater(self._delaytime, self.next)

    def next(self):
        """
        Reset the timer and show the next image. If we reached the end the
        timer is automatically stopped. If you add images or go backwards, you
        may want to use start to start it again.
        """
        if not self.index < self.length - 2 :
            self._active = False
        self._reset_timer()
        Slideshow.next(self)

    def previous(self):
        """
        Reset the timer and show the previous image again
        """
        self._reset_timer()
        Slideshow.previous(self)



if __name__ == "__main__":
    import pgm
    import os, gobject, sys

    def next(show):
        print "next"
        if show.get_index() < show.get_length():
            show.next()
            return True

    def on_delete(dock,event):
        loop.quit()

    def on_key_pressed(port, event, dock):
            if event.type == pgm.KEY_PRESS:
                if event.keyval == pgm.keysyms.Escape or \
                    event.keyval == pgm.keysyms.q:
                    loop.quit()

#                elif event.keyval == pgm.keysyms.Down:
#                    dock.volume = dock.volume - 10
#
#                elif event.keyval == pgm.keysyms.Up:
#                    dock.volume = dock.volume + 10
#
                elif event.keyval == pgm.keysyms.Right:
                   show.next()

                elif event.keyval == pgm.keysyms.Left:
                    show.previous()
#
#                elif event.keyval == pgm.keysyms.s:
#                    dock.time_layout = pld.TIME_LAYOUT_NORMAL
#
#                elif event.keyval == pgm.keysyms.d:
#                    dock.time_layout = pld.TIME_LAYOUT_MINUS
##
 #               elif event.keyval == pgm.keysyms.f:
 #                   dock.time_layout = pld.TIME_LAYOUT_ELAPSED
#
#                elif event.keyval == pgm.keysyms.i:
#                    dock.playing_length = 1000000
#
#                elif event.keyval == pgm.keysyms.p:
#                    dock.playing_length = 10
#
#                elif event.keyval == pgm.keysyms.o:
#                    dock.playing_length = 3700
#
            return True

    factory = pgm.ViewportFactory('opengl')
    gl = factory.create()
    gl.title = 'Slideshow widget'

    canvas = pgm.Canvas()

    gl.set_canvas(canvas)

    list = ["examples/pictures/fluendo.png" for i in range(5)]
    show = Slideshow(canvas, list)

#    dock.title = "This is a playing test"

#    dock.time = 50
#    dock.time_bar_bg_color = (75, 91, 14, 255)
#    dock.time_bar_fg_color  = 153, 197, 18, 255
#    dock.time = 5
#    dock.conversion_string = ""
    loop = gobject.MainLoop()
    gobject.timeout_add(5, gl.update)
    gobject.timeout_add(10000, next, show)
    gl.connect('delete-event', on_delete)
    gl.connect('key-press-event', on_key_pressed, show)
    gl.show()
#    gobject.timeout_add(15, dock, gl, loop)
    loop.run()
